module Main where

import Graphics.Gloss
import System.Random

          --((x,y), (vx, vy), p)
type Bola = ((Float, Float), (Float, Float), Picture)

desenhaBola :: Bola -> Picture
desenhaBola ((x, y), _, p) =
  translate x y p

desenhaBolas :: [Bola] -> Picture
desenhaBolas bs =
  pictures $ map desenhaBola bs

atualizaPosicao :: Float -> Bola -> Bola
atualizaPosicao dt ((x, y), v@(vx, vy), p) =
  ((x + vx * dt, y + vy * dt), v, p)

atualizaVelocidades :: Bola -> Bola
atualizaVelocidades b@((x, y), (vx, vy), p)
  | x > lx  = (( lx, y), (-vx, vy), p)
  | x < -lx = ((-lx, y), (-vx, vy), p)
  | y > ly  = ((x,  ly), ( vx, -vy), p)
  | y < -ly = ((x, -ly), (vx, -vy), p)
  | otherwise   = b
  where
    lx = limiteX - raio
    ly = limiteY - raio

limiteX, limiteY, raio :: Num a => a
limiteX = 400
limiteY = 300
raio = 55 -- Fixado pelo tamanho do bitmap. Poder-se-ia tambem obter o
          -- tamanho diretamente do bitmap.

atualizaMundo :: p -> Float -> [Bola] -> [Bola]
atualizaMundo _ dt = map (atualizaVelocidades . atualizaPosicao dt)


criaBola :: (Float, Float) -> (Float, Float) -> Picture -> Bola
criaBola coord v p = (coord, v, p)

criaBolas :: StdGen -> Int -> [Picture] -> [Bola]
criaBolas g qtd pics =
  take qtd $ zipWith3 criaBola coords vels pics
  where
    (pGen, angGen) = split g
    (xGen, yGen) = split pGen
    vel = 200
    convAng ang = (vel * cos ang, vel * sin ang)
    vels = map (\x -> convAng $ x * 2 * pi) (randoms angGen)

    convCoord x y = (x * 2 * limiteX - limiteX, y * 2 * limiteY - limiteY)
    coords = zipWith convCoord (randoms xGen) (randoms yGen)

figuras :: [String]
figuras = ["emilio.bmp", "hal.bmp", "lena.bmp", "olivetti.bmp", "ufabc.bmp"]

main :: IO ()
main = do

  g <- getStdGen
  figs <- mapM loadBMP figuras
  let bolas = criaBolas g 5 (cycle figs)

  simulate
    janela
    black
    60
    bolas
    desenhaBolas
    atualizaMundo
  where
    janela = InWindow "Paredao" (2 * limiteX, 2 * limiteY) (50, 50)
